@@ -1,13 +1,15 @@
 package Test::POE::Server::TCP;
+BEGIN {
+  $Test::POE::Server::TCP::VERSION = '1.12';
+}
+
+# ABSTRACT: A POE Component providing TCP server services for test cases
 
 use strict;
 use warnings;
 use POE qw(Wheel::SocketFactory Wheel::ReadWrite Filter::Line);
 use Socket;
 use Carp qw(carp croak);
-use vars qw($VERSION);
-
-$VERSION = '1.08';
 
 sub spawn {
   my $package = shift;
@@ -26,6 +28,7 @@ sub spawn {
 		      send_to_all_clients => '_send_to_all_clients',
 		      disconnect          => '_disconnect',
 		      terminate           => '_terminate',
+          start_listener      => '_start_listener',
 	            },
 	   $self => [ qw(_start register unregister _accept_client _accept_failed _conn_input _conn_error _conn_flushed _conn_alarm _send_to_client __send_event _disconnect _send_to_all_clients) ],
 	],
@@ -89,6 +92,19 @@ sub _start {
     $kernel->detach_myself();
   }
 
+  $kernel->call( $self->{session_id}, 'start_listener' );
+  return;
+}
+
+sub start_listener {
+  my $self = shift;
+  $poe_kernel->call( $self->{session_id}, 'start_listener', @_ );
+}
+
+sub _start_listener {
+  my ($kernel,$self) = @_[KERNEL,OBJECT];
+  return if $self->{listener};
+
   $self->{listener} = POE::Wheel::SocketFactory->new(
       ( defined $self->{address} ? ( BindAddress => $self->{address} ) : () ),
       ( defined $self->{port} ? ( BindPort => $self->{port} ) : ( BindPort => 0 ) ),
@@ -223,7 +239,7 @@ sub _test_filter {
 sub _accept_failed {
   my ($kernel,$self,$operation,$errnum,$errstr,$wheel_id) = @_[KERNEL,OBJECT,ARG0..ARG3];
   warn "Wheel $wheel_id generated $operation error $errnum: $errstr\n";
-  delete $self->{listener};
+  delete $self->{listener} if $operation eq 'listen';
   $self->_send_event( $self->{_prefix} . 'listener_failed', $operation, $errnum, $errstr );
   return;
 }
@@ -264,8 +280,9 @@ sub _conn_input {
 sub _conn_error {
   my ($self,$errstr,$id) = @_[OBJECT,ARG2,ARG3];
   return unless $self->_conn_exists( $id );
-  delete $self->{clients}->{ $id };
-  $self->_send_event( $self->{_prefix} . 'disconnected', $id );
+  my $href = delete $self->{clients}->{ $id };
+  delete $href->{wheel};
+  $self->_send_event( $self->{_prefix} . 'disconnected', $id,  map { $href->{$_} } qw(peeraddr peerport sockaddr sockport) );
   return;
 }
 
@@ -387,11 +404,6 @@ sub __send_event {
   return;
 }
 
-#sub send_event {
-#  my $self = shift;
-#  $poe_kernel->post( $self->{session_id}, '__send_event', @_ );
-#}
-
 sub _send_event  {
   my $self = shift;
   my ($event, @args) = @_;
@@ -441,12 +453,18 @@ sub _send_to_all_clients {
 
 q{Putting the test into POE};
 
+
 __END__
+=pod
 
 =head1 NAME
 
 Test::POE::Server::TCP - A POE Component providing TCP server services for test cases
 
+=head1 VERSION
+
+version 1.12
+
 =head1 SYNOPSIS
 
 A very simple echo server with logging of requests by each client:
@@ -645,7 +663,7 @@ communication with the component when sending data to the client or disconnectin
 
 =head1 CONSTRUCTOR
 
-=over 
+=over
 
 =item C<spawn>
 
@@ -732,6 +750,10 @@ Access to the L<POE::Wheel::SocketFactory> method of the underlying listening so
 
 Returns the port that the component is listening on.
 
+=item C<start_listener>
+
+If the listener fails on C<listen> you can attempt to restart it with this.
+
 =back
 
 =head1 INPUT EVENTS
@@ -760,7 +782,6 @@ Send some output to a connected client. First parameter must be a valid client i
 The second parameter may also be an arrayref of items to send to the client. If the filter you have used requires an arrayref as
 input, nest that arrayref within another arrayref.
 
-
 =item C<send_to_all_clients>
 
 Send some output to all connected clients. The parameter is a string of text to send.
@@ -775,6 +796,10 @@ Places a client connection in pending disconnect state. Requires a valid client
 
 Immediately disconnects a client conenction. Requires a valid client ID as a parameter.
 
+=item C<start_listener>
+
+If the listener fails on C<listen> you can attempt to restart it with this.
+
 =back
 
 =head1 OUTPUT EVENTS
@@ -794,6 +819,9 @@ Generated if the component cannot either start a listener or there is a problem
 accepting client connections. ARG0 contains the name of the operation that failed. 
 ARG1 and ARG2 hold numeric and string values for $!, respectively.
 
+If the operation was C<listen>, the component will remove the listener.
+You may attempt to start it again using C<start_listener>.
+
 =item C<testd_connected>
 
 Generated whenever a client connects to the component. ARG0 is the client ID, ARG1
@@ -802,12 +830,14 @@ ARG4 is our socket port.
 
 =item C<testd_disconnected>
 
-Generated whenever a client disconnects. ARG0 is the client ID.
+Generated whenever a client disconnects. ARG0 was the client ID, ARG1
+was the client's IP address, ARG2 was the client's TCP port. ARG3 was our IP address and
+ARG4 was our socket port.
 
 =item C<testd_client_input>
 
-Generated whenever a client sends us some traffic. ARG0 is the client ID, ARG1 is the data sent ( tokenised by whatever POE::Filter you 
-specified. 
+Generated whenever a client sends us some traffic. ARG0 is the client ID, ARG1 is the data sent 
+( tokenised by whatever POE::Filter you specified ).
 
 =item C<testd_client_flushed>
 
@@ -815,17 +845,9 @@ Generated whenever anything we send to the client is actually flushed down the '
 
 =back
 
-=head1 AUTHOR
+=head1 CREDITS
 
-Chris C<BinGOs> Williams <chris@bingosnet.co.uk>
-
-with code borrowed from L<POE::Component::Server::TCP> by Rocco Caputo, Ann Barcomb and Jos Boumans.
-
-=head1 LICENSE
-
-Copyright E<copy> Chris Williams, Rocco Caputo, Ann Barcomb and Jos Boumans.
-
-This module may be used, modified, and distributed under the same terms as Perl itself. Please see the license that came with your Perl distribution for details.
+This module uses code borrowed from L<POE::Component::Server::TCP> by Rocco Caputo, Ann Barcomb and Jos Boumans.
 
 =head1 SEE ALSO
 
@@ -833,4 +855,16 @@ L<POE>
 
 L<POE::Component::Server::TCP>
 
+=head1 AUTHOR
+
+Chris Williams <chris@bingosnet.co.uk>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Chris Williams, Rocco Caputo, Ann Barcomb and Jos Boumans.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
+
